/********************************************************************
Beispiel fr RFM02-Funkmodul (Senden)
IAR Embedded Workbench IDE, C
Prozessor: ATMEGA8
Frequenz: 434MHz
Datenrate: 4.8kbps

Diese Programm stellt eine kurze Testroutine fr den ATMEGA8 in
Verbindung mit einem externen 8MHz Quarz, einem RFM02-Funkmodul, und
dem Pollin Electronic Funk-AVR-Evaluations-Board dar.
Es handelt sich ausdrcklich um eine Testroutine und ein Anwendungs-
beispiel. Elementare Elemente wie Fehlerbehandlungen usw. sind nicht
vorhanden.

Programmablauf:
Sendet Preamble, Frame-Recognition, 16 Datenbyte und ChkSumme der
16 Datenbyte. Anschlieend erfolgt eine Signalisierung mit LED1.
Dies geschieht in einer Endlosschleife.
-----------------------
ATMEGA8           RFM02
-----------------------
SCK               SCK
MISO              N.C.
MOSI              SDI
SS                nSEL
INT0              nIRQ
PD7               FSK
PD5               LED2
PD6               LED1
PD4               Module-Power
*********************************************************************/

#include <iom8.h>
#define DDR_IN 0
#define DDR_OUT 1
#define DDR_LED DDRD
#define PORT_LED PORTD
#define PORT_SEL PORTB
#define PIN_SEL PINB
#define DDR_SEL DDRB
#define PORT_SDI PORTB
#define PIN_SDI PINB
#define DDR_SDI DDRB
#define PORT_SCK PORTB
#define PIN_SCK PINB
#define DDR_SCK DDRB
#define PORT_SDO PORTB
#define PIN_SDO PINB
#define DDR_SDO DDRB
#define PB7 7
#define PB6 6// |
#define RFXX_SCK 5// |
#define RFXX_SDO 4// |RF_PORT
#define RFXX_SDI 3// |
#define RFXX_SEL 2// |
#define PB1 1//
#define PB0 0//
#define RFXX_DATA 7//
#define SEL_OUTPUT() DDR_SEL |= (1<<RFXX_SEL)
#define HI_SEL() PORT_SEL|= (1<<RFXX_SEL)
#define LOW_SEL() PORT_SEL&=~(1<<RFXX_SEL)
#define SDI_OUTPUT() DDR_SDI |= (1<<RFXX_SDI)
#define HI_SDI() PORT_SDI|= (1<<RFXX_SDI)
#define LOW_SDI() PORT_SDI&=~(1<<RFXX_SDI)
#define SDO_INPUT() DDR_SDO&= ~(1<<RFXX_SDO)
#define SDO_HI() PIN_SDO&(1<<RFXX_SDO)
#define SCK_OUTPUT() DDR_SCK |= (1<<RFXX_SCK)
#define HI_SCK() PORT_SCK|= (1<<RFXX_SCK)
#define LOW_SCK() PORT_SCK&=~(1<<RFXX_SCK)
#define LED_OUTPUT() DDR_LED|=((1<<6)|(1<<5)|(1<<7)|(1<<4))//+FSKPIN+Module-Power
#define LED1_ON() PORT_LED|=(1<<6)
#define LED1_OFF() PORT_LED&=~(1<<6)
#define MODULE_ON() PORT_LED|=(1<<4)
#define MODULE_OFF() PORT_LED&=~(1<<4)
#define LED1_TRG() PORT_LED^=(1<<6)

void Delay_ms(unsigned char amS){
  unsigned char i;
  unsigned int j;
  for(i=0;i<amS;i++)for(j=0;j<914;j++);
}

void RFXX_PORT_INIT(void){
  HI_SEL();
  HI_SDI();
  LOW_SCK();
  SEL_OUTPUT();
  SDI_OUTPUT();
  SDO_INPUT();
  SCK_OUTPUT();
}

unsigned int RFXX_WRT_CMD(unsigned int aCmd){
  unsigned char i;
  unsigned int temp;
  LOW_SCK();
  LOW_SEL();
  for(i=0;i<16;i++){
    temp<<=1;
    if(SDO_HI()){
      temp|=0x0001;
    }
    LOW_SCK();
    if(aCmd&0x8000){
      HI_SDI();
      }else{
      LOW_SDI();
    }
    HI_SCK();
    aCmd<<=1;
  };
  LOW_SCK();
  HI_SEL();
  return(temp);
}

void RF02B_SEND(unsigned char aByte){
  unsigned char i;
  for(i=0;i<8;i++){
    while(PIND&(1<<2));//Polling nIRQ
    while(!(PIND&(1<<2)));
    if(aByte&0x80){
      PORTD|=(1<<RFXX_DATA);
      }else{
      PORTD&=~(1<<RFXX_DATA);
    }
    aByte<<=1;
  }
}

int main(void)
{
  unsigned int i,j,ChkSum;

  LED_OUTPUT();
  MODULE_OFF(); //for reset
  LED1_ON();
  Delay_ms(200);
  LED1_OFF();
  Delay_ms(200);
  LED1_ON();
  Delay_ms(200);
  LED1_OFF();
  Delay_ms(200);
  LED1_ON();
  Delay_ms(200);
  LED1_OFF();
  Delay_ms(200);
  Delay_ms(200);
  Delay_ms(200);
  Delay_ms(200);
  MODULE_ON(); //for reset
  Delay_ms(200);

  RFXX_PORT_INIT();
  RFXX_WRT_CMD(0xCC00);
  RFXX_WRT_CMD(0x8B61);//+/-90kHz
  RFXX_WRT_CMD(0xA640);//434MHz
  RFXX_WRT_CMD(0xD040);//RATE/2
  RFXX_WRT_CMD(0xC823);//4.8kbps
  RFXX_WRT_CMD(0xC220);//ENABLE BIT SYNC
  RFXX_WRT_CMD(0xC001);//CLOSE ALL

  PORTB|=(1<<RFXX_DATA);
  DDRB|=(1<<RFXX_DATA);//SET DATA OUTPUT
  while(1){
    RFXX_WRT_CMD(0xC039);//START TX
    ChkSum=0;
    RF02B_SEND(0xAA);//PREAMBLE
    RF02B_SEND(0xAA);//PREAMBLE
    RF02B_SEND(0xAA);//PREAMBLE
    RF02B_SEND(0x2D);//HEAD HI BYTE
    RF02B_SEND(0xD4);//HEAD LOW BYTE
    RF02B_SEND(0x30);//DATA0
    ChkSum+=0x30;
    RF02B_SEND(0x31);//DATA1
    ChkSum+=0x31;
    RF02B_SEND(0x32);
    ChkSum+=0x32;
    RF02B_SEND(0x33);
    ChkSum+=0x33;
    RF02B_SEND(0x34);
    ChkSum+=0x34;
    RF02B_SEND(0x35);
    ChkSum+=0x35;
    RF02B_SEND(0x36);
    ChkSum+=0x36;
    RF02B_SEND(0x37);
    ChkSum+=0x37;
    RF02B_SEND(0x38);
    ChkSum+=0x38;
    RF02B_SEND(0x39);
    ChkSum+=0x39;
    RF02B_SEND(0x3A);
    ChkSum+=0x3A;
    RF02B_SEND(0x3B);
    ChkSum+=0x3B;
    RF02B_SEND(0x3C);
    ChkSum+=0x3C;
    RF02B_SEND(0x3D);
    ChkSum+=0x3D;
    RF02B_SEND(0x3E);
    ChkSum+=0x3E;
    RF02B_SEND(0x3F);//DATA15
    ChkSum+=0x3F;
    RF02B_SEND(ChkSum);//DATA16
    RF02B_SEND(0xAA);//DUMMY BYTE
    RFXX_WRT_CMD(0xC001);//CLOSE TX
    LED1_ON();
    Delay_ms(200);
    LED1_OFF();
    for(i=0;i<5000;i++)for(j=0;j<123;j++);
  };
}